package week04

import org.scalatest.concurrent.{Signaler, TimeLimitedTests}
import org.scalatest.flatspec.AnyFlatSpec
import org.scalatest.time.{Millis, Span}
import ZH._

import scala.util.Random

class ZHTest extends AnyFlatSpec with TimeLimitedTests {
  val timeLimit: Span = Span(1000, Millis)
  override val defaultTestSignaler: Signaler = new Signaler {
    override def apply(testThread: Thread): Unit = {
      println("Ez a teszt túl sokáig fut.")
      testThread.stop()
    } //unsafe, never használd.
  }

  def generateIntList(length: Int, range: Int, positives: Boolean): List[Int] = Random.nextInt(length) match {
    case 0 => Nil
    case _ => {
      val head = Random.nextInt(range);
      (if (positives) head else (head - range / 2)) :: generateIntList(length, range, positives)
    }
  }

  def generateAtlagTeszt: (List[Double], Double) = Random.nextInt(10) match {
    case 0 => (Nil,0.0)
    case _ => {
      val head = Random.nextDouble()
      val (list,sum) = generateAtlagTeszt
      (head::list, sum + head)
    }
  }
  "atlag" should "compute the average of the list" in {
    for (i <- 1 to 1000) {
      val (list,sum) = generateAtlagTeszt
      val avg = list match {
        case Nil => Double.NaN
        case _ => sum/list.length
      }
      val result = atlag(list)
      assert ((result.isNaN && avg.isNaN) || Math.abs(atlag(list) - avg) < 1e-06, "teszteset: " + list)
    }
  }

  def generateCsuszoatlag2Teszt: (List[Double], List[Double]) = Random.nextInt(20) match {
    case 0 => (Nil,Nil)
    case _ => {
      val (inputTail, outputTail) = generateCsuszoatlag2Teszt
      val head = Random.nextDouble()
      inputTail match {
        case Nil => (List(head),Nil)
        case head2 :: _ => (head::inputTail, (head+head2)/2.0 :: outputTail)
      }
    }
  }
  "csuszoatlag2" should "compute the sliding average of the list, with a window size of 2" in {
    for( i <- 1 to 1000) {
      val (input, output) = generateCsuszoatlag2Teszt
      assert (csuszoatlag2(input) == output , "teszteset: " + input)
    }
  }

  def generateCsuszoAtlagGeneralTeszt(size: Int): (List[Double],List[Double]) = {
    def collectWindow(list: List[Double], size: Int): Option[Double] = if(size==0) Some(0.0) else list match {
      case Nil => None
      case head::tail => collectWindow(tail, size-1).flatMap( d => Some( d+head ))
    }
    def sublists(list: List[Double]): List[Double] = list match {
      case Nil => Nil
      case head::tail => {
        val suffix = collectWindow(tail, size-1)
        suffix match {
          case None => Nil
          case Some(sum) => ((sum+head)/size)::sublists(tail)
        }
      }
    }
    def generate(length: Int): List[Double] =
      if(length==0) Nil else Random.nextDouble()::generate(length-1)
    val input = generate(Random.nextInt(size) + size)
    (input, sublists(input))
  }
  "csuszoatlagGeneral" should "compute the sliding average of the list, with a parametric window size" in {
    for (i <- 1 to 1000) {
      val size = Random.nextInt(5) + 2
      val (input,output) = generateCsuszoAtlagGeneralTeszt(size)
      val result = ZH.csuszoatlagGeneral(input,size)
      assert( (result.length == output.length) && result.zip(output).map { p => Math.abs( p._1 - p._2 ) }.sum < 1e-06,  "teszteset: list = " + input + " size = " + size)
    }
  }

  "testPrefix" should "check whether the input list has the input prefix" in {
    for(i <- 1 to 1000) {
      val list = Random.nextInt().toBinaryString
      val prefix = Random.nextInt(10).toBinaryString
      assert( testPrefix(list.toList, prefix.toList) == list.startsWith(prefix), "teszteset: list = " + list + ", prefix = " + prefix)
    }
  }

  "containsNeedle" should "check whether haystack contains needle somewhere" in {
    for(i <- 1 to 1000) {
      val list = Random.nextInt().toBinaryString
      val prefix = Random.nextInt(50).toBinaryString
      assert( containsNeedle(list.toList, prefix.toList) == list.contains(prefix), "teszteset: haystack = " + list + ", needle = " + prefix)
    }
  }
}